#include "animations.h" // includes global.h

#include "components.h"
#include "map.h"

#define GATE_CLOSED		-2
#define GATE_CLOSING	-1
#define GATE_OPENING	1
#define GATE_OPENED		2

#define PIG_MEMORY		100 // how many steps the pig can plan forward
#define PIG_MAX_SPEED	10
#define PIG_DEF_SPEED	2

#define WELLARM_MINROT -20
#define WELLARM_MAXROT 65
#define WELLARM_MINBOUND -50
#define WELLARM_MAXBOUND 120

boolean			pigMoving			= FALSE;		// is the pig moving
int				pigMovementPlans	= 0;			// what the pig wants to do: negative: wait, positive: move forward. The value shows the remaining rounds.
unsigned int	pigStep				= PIG_DEF_SPEED;// how fast the pig goes
int				pigRotate			= -5;			// how many degrees the pig rotates in one step
static int		degreesLeft			= 0;			// degrees left of the current rotation task of the pig
boolean			pigRotating			= FALSE;		// is the pig rotating now
int				pigDirection		= 0;			// which direction the pig is looking at now. Only to help movePig()
int				gateAction			= GATE_CLOSED;	// -2 - closed, -1 - closing, 1 - opening, 2 - opened
unsigned int	gateRotate			= 10;			// how many degrees the gate rotates in one step (abs value)
double			wellArmRot			= 45;			// the current rotation of the wellarm
double			wellArmDelta		= -5;			// the delta rotation of the wellarm in the next round

void setDefSpeeds() {
	pigStep = PIG_DEF_SPEED;
	pigRotate = -5;
	gateRotate = 10;
}

void setDefStates() {
	pigMoving = FALSE;
	pigMovementPlans = 0;
	pigRotating = FALSE;
	degreesLeft = 0;
	gateAction = GATE_CLOSED;
}

void setDefPositions() {
	pigDirection = 0;
}

/* INTERFACE
 * reset everything
 */
void resetAnimations() {
	setDefSpeeds();
	setDefStates();
	setDefPositions();
}

/* INTERFACE
 * get the pig going
 */
void startPig() {
	pigMoving = TRUE;
	setDefSpeeds();
	nudgePig();
}

/* INTERFACE
 * stop the pig
 */
void stopPig() {
	pigMoving = FALSE;
	pigMovementPlans = 0;
}

/* INTERFACE
 * nudge piggie to move faster. Will reset once it stops.
 */
void nudgePig() {
	pigMovementPlans = PIG_MEMORY;
	if(pigStep < PIG_MAX_SPEED)
		pigStep++;
		
	pigRotate *= ((float)pigStep / PIG_DEF_SPEED);
}

void planPigMovements() {
	int i;

	// get random number; if small, add some, negate and make wait; if big, subtract some and make go
	srand(time(NULL));	
	i = rand() % (PIG_MEMORY + 1);
	
	if(i <= PIG_MEMORY / 2) {
		i += PIG_MEMORY / 4;
		i *= -1;
	} else {
		i -= PIG_MEMORY / 4;
	}
	
	pigMovementPlans = i;
	
}

/*
 * return if the pig can move forward while staying inside bounds.
 */
int canMovePig() {
	int nextX = pig.properties.position.x + FIELDSIZE/2.0, nextZ = pig.properties.position.z + FIELDSIZE/2.0;

#ifdef DEBUG_PIGPATH	
	printf("\n\n==============================\ncanMovePig output:\n\npig.properties.position.x: %d, scaled:%d, pig.properties.position.z: %d, scaled:%d\n", (int)pig.properties.position.x, nextX, (int)pig.properties.position.z, nextZ);
#endif
	
	switch(pigDirection) {
		case 0: // east
			nextX += (pigStep + 30);
			break;
		case 90: // north
			nextZ -= (pigStep + 30);
			break;
		case 180: // west
			nextX -= (pigStep + 30);
			break;
		case 270: // south
			nextZ += (pigStep + 30);
			break;
	}
	
#ifdef DEBUG_PIGPATH	
	printf("nextX: %d, nextZ:%d, bounds[%d][%d]: %d\n\n==============================\n\n", nextX, nextZ, nextX/AREASIZE, nextZ/AREASIZE, bounds[nextX/AREASIZE][nextZ/AREASIZE]);
#endif	
	
	return bounds[nextX/AREASIZE][nextZ/AREASIZE];
}

/*
 * returns true if rotation is complete
 */
boolean rotatePig() {
	int lPigRotate = pigRotate;
	
	pigRotating = TRUE;
	
	if(!degreesLeft)
		degreesLeft = 90 * (lPigRotate < 0 ? -1 : 1); // good for both directions
		
	if(abs(degreesLeft) < abs(lPigRotate)) {
		lPigRotate = degreesLeft;
	}
	
	pig.properties.rotation.yAxis += lPigRotate;
	
	degreesLeft -= lPigRotate;
	pigDirection += lPigRotate;
	
	if(pigDirection < 0)
		pigDirection += 360;
	else if(pigDirection >= 360)
		pigDirection -= 360;
	
	if(!degreesLeft)
		return TRUE;
	else
		return FALSE;
}

void movePig() {
	if(++pigMovementPlans > 0) { // this solves the waitings too
		if(canMovePig()) {
			switch(pigDirection) {
				case 0: // east
					pig.properties.position.x += pigStep;
					break;
				case 90: // north
					pig.properties.position.z -= pigStep;
					break;
				case 180: // west
					pig.properties.position.x -= pigStep;
					break;
				case 270: // south
					pig.properties.position.z += pigStep;
					break;
			}
			pigMovementPlans -= 2; // we have incremented this in the outer if, so now have to subtract 2
		} else { // no decrementing here, thus the pig will move some more after it finished rotating
			rotatePig();
		}
	}
}

/*
 * returns if the pig is in the path of the moving gates
 */
boolean pigInWay() {
	return ((pig.properties.position.x >= gate_l.properties.position.x-60 && pig.properties.position.x <= gate_r.properties.position.x+60) && (pig.properties.position.z >= gate_l.properties.position.z-30 && pig.properties.position.z <= gate_l.properties.position.z+90));
}

/* INTERFACE
 * opens the gates. The pig can move through.
 */
void openGate() {
	if(pigInWay())
		return;
	else
		gateAction = GATE_OPENING;
		
	setBoundsOpenGates(); // map.h
}

/* INTERFACE
 * closes the gates. The pig can't move through.
 */
void closeGate() {
	if(pigInWay())
		return;
	else
		gateAction = GATE_CLOSING;
	
	setBoundsClosedGates(); // map.h
}

/*
 * moves the gates one step to the direction set in gateAction. Returns true if complete.
 */
boolean moveGates() {

	switch(gateAction) {
		case GATE_CLOSING:
		
			gate_l.properties.rotation.yAxis += gateRotate;
			gate_r.properties.rotation.yAxis -= gateRotate;
			
			if(gate_l.properties.rotation.yAxis >= gatepart.properties.rotation.yAxis) {
				gate_l.properties.rotation.yAxis = gatepart.properties.rotation.yAxis;
				gate_r.properties.rotation.yAxis = gatepart.properties.rotation.yAxis+180.0;
				return TRUE;
			} else {
				return FALSE;
			}
			break;
			
		case GATE_OPENING:
		
			gate_l.properties.rotation.yAxis -= gateRotate;
			gate_r.properties.rotation.yAxis += gateRotate;
			
			if(gate_l.properties.rotation.yAxis <= gatepart.properties.rotation.yAxis-100.0) {
				gate_l.properties.rotation.yAxis = gatepart.properties.rotation.yAxis-100.0;
				gate_r.properties.rotation.yAxis = gatepart.properties.rotation.yAxis-80.0;
				return TRUE;
			} else {
				return FALSE;
			}
			break;
	}
	
	return TRUE;
}

/*
 * moves the wellarm
 */
void moveWellArm() {
	wellArmRot += wellArmDelta;
	
	if(wellArmRot >= WELLARM_MAXBOUND || wellArmRot <= WELLARM_MINBOUND) {
		wellArmDelta = -wellArmDelta;
	}
	
	if(wellArmRot > WELLARM_MAXROT) {
		wellarm.properties.rotation.xAxis = WELLARM_MAXROT;
	} else if(wellArmRot < WELLARM_MINROT) {
		wellarm.properties.rotation.xAxis = WELLARM_MINROT;
	} else {
		wellarm.properties.rotation.xAxis = wellArmRot;
	}
}

/* INTERFACE
 * progress every ongoing animations by 1 step
 */
void Tick() {
	if(pigMoving) {
		if(pigMovementPlans == 0) {
			planPigMovements();
			setDefSpeeds();
		}
			
		if(pigRotating) {
			if(rotatePig())
				pigRotating = FALSE;
		} else {
			movePig();
		}
	}
	
	if(gateAction == GATE_CLOSING) {
		if(moveGates())
			gateAction = GATE_CLOSED;
	} else if(gateAction == GATE_OPENING) {
		if(moveGates())
			gateAction = GATE_OPENED;
	}
	
	moveWellArm();
}
