#include "components.h"

#include "map.h"
#include "drawing.h"

/*
 * top level objects
 */
#define OBJ_FIELD		0x00000
#define OBJ_PIG			0x10000
#define OBJ_HOUSE		0x20000
#define OBJ_FENCEPART	0x30000
#define OBJ_GATEPART	0x40000
#define OBJ_CARRIAGE	0x50000
#define OBJ_WELLARM		0x60000
#define OBJ_WELLPOLE	0x70000
#define OBJ_WELLPIT		0x80000
#define OBJ_TROUGH		0x90000
/*
 * Components
 *
 * UNUSED, only left here to show the list of objects
 */
/*
 * pig components
 */
#define OBJ_S_PIG_EYE		0x11000
#define OBJ_S_PIG_NOSE		0x12000
#define OBJ_S_PIG_HEAD		0x13000
#define OBJ_S_PIG_TAIL		0x14000
#define OBJ_S_PIG_BODY		0x15000
#define OBJ_S_PIG_LEG		0x16000
#define OBJ_S_PIG_EAR		0x17000
/*
 * house components
 */
#define OBJ_S_HOUSE_ROOF	0x21000
#define OBJ_S_HOUSE_BODY	0x22000
/*
 * house body components
 */
#define OBJ_SS_HOUSE_BOD_WNDP	0x22100
#define OBJ_SS_HOUSE_BOD_DOORP	0x22200
#define OBJ_SS_HOUSE_BOD_STDP	0x22300
#define OBJ_SS_HOUSE_BOD_PORCHP	0x22400
/*
 * house body porch components
 */
 #define OBJ_SSS_HOUSE_BOD_PCHP_PILLAR	0x22400
/*
 * fence components
 */
#define OBJ_S_FENCE_POLE	0x31000
#define OBJ_S_FENCE_BAR		0x32000
/*
 * gate components
 */
#define OBJ_S_GATE_POLE		0x41000
#define OBJ_S_GATE_BAR		0x42000
/*
 * carriage components
 */
#define OBJ_S_CAR_BODY		0x51000
#define OBJ_S_CAR_WHEEL		0x52000
/*
 * carriage wheel components
 */
#define OBJ_SS_CAR_WHEEL_SPOKE	0x52000 //HU:kll
/*
 * wellarm components
 */
#define OBJ_S_WLLARM_ARM	0x61000
#define OBJ_S_WLLARM_BUCKET	0x62000

/*
 * "what" definitions for "what do you want to have on this piece of wall?"
 */
#define NOTHING	0
#define WINDOW	1
#define DOOR	2
#define PORCH	3

/*
 * Colors
 */
#define COLOR_WHITE		1.0,  1.0,  1.0
#define COLOR_BLACK		0.0,  0.0,  0.0

#define COLOR_FIELD		0.0,  0.35, 0.0
#define COLOR_PIG_BODY	1.0,  0.6,  0.6
#define COLOR_PIG_EYE	0.0,  0.0,  0.0
#define COLOR_WALL		1.0,  1.0,  1.0
#define COLOR_GLASS		0.13, 0.26, 1.0
#define COLOR_PNT_WOOD	0.5,  0.3,  0.3
#define COLOR_STD_WOOD	0.6,  0.3,  0.15
#define COLOR_METAL		0.15, 0.15, 0.15
#define COLOR_FLOOR		0.25, 0.25, 0.25
#define COLOR_ROOF		0.6,  0.15, 0.0
#define COLOR_WATER		0.15, 0.3,  1.0

/* INTERFACE
 * interface vars
 */
//                     ID                position   rotation   size(h, w, l)
Object field		= {OBJ_FIELD,		{{0, 0, 0}, {0, 0, 0}, {50,  FIELDSIZE,  FIELDSIZE }} };
Object pig			= {OBJ_PIG,			{{0, 0, 0}, {0, 0, 0}, {30,  30,  60 }} };
Object house		= {OBJ_HOUSE,		{{0, 0, 0}, {0, 0, 0}, {310, 1090,790}} };
Object fencepart	= {OBJ_FENCEPART,	{{0, 0, 0}, {0, 0, 0}, {50,  150, 10 }} };
Object gatepart		= {OBJ_GATEPART,	{{0, 0, 0}, {0, 0, 0}, {50,  75,  5  }} };
Object carriage		= {OBJ_CARRIAGE,	{{0, 0, 0}, {0, 0, 0}, {100, 120, 250}} };
Object wellarm		= {OBJ_WELLARM,		{{0, 0, 0}, {0, 0, 0}, {5,   5,   160}} };
Object wellpole		= {OBJ_WELLPOLE,	{{0, 0, 0}, {0, 0, 0}, {95,  5,   5  }} };
Object wellpit		= {OBJ_WELLPIT,		{{0, 0, 0}, {0, 0, 0}, {200, 75,  75 }} };
Object trough		= {OBJ_TROUGH,		{{0, 0, 0}, {0, 0, 0}, {20,  30,  100}} };

Object fence[FENCECOUNT]; // init in map.h/buildMap()!
Object gate_l; // init in map.h/buildMap()!
Object gate_r; // init in map.h/buildMap()!

double windowAlpha = 0.2;
double waterAlpha = 0.4;

boolean spotLightsOn = FALSE;
boolean sunOn = FALSE;

/*
 * non-interface function declarations
 */
void putFieldBy(Properties properties);
void putPigBy(Properties properties);
void putHouseBy(Properties properties);
void putFencepartBy(Properties properties);
void putGatepartBy(Properties properties);
void putCarriageBy(Properties properties);
void putWellarmBy(Properties properties);
void putWellpoleBy(Properties properties);
void putWellpitBy(Properties properties);
void putTroughBy(Properties properties);

void putWallStdWith(int what); // see defined "whats" at about 60 lines above

/***************************************************************************
 *** TRANSPARENT STUFF MANAGEMENT ******************************************
 **************************************************************************/
 
typedef struct wndLst {
	GLdouble * modelMatrix;
	struct wndLst * next;
} windowList;

windowList * windows = NULL;
windowList * pWindow;

void addWindow(GLdouble * windowModelMatrix) {
	pWindow = (windowList *)malloc(sizeof(windowList));
	pWindow->modelMatrix = windowModelMatrix;
	pWindow->next = windows;
	windows = pWindow;
}

void clearWindows() {
	while(windows != NULL) {
		pWindow = windows->next;
		free(windows->modelMatrix);
		free(windows);
		windows = pWindow;
	}
}

void paintWindows() {
	
	glPushMatrix();
		pWindow = windows;
		
		while(pWindow != NULL) {
			glLoadMatrixd(pWindow->modelMatrix);
			
			Size s;
			
			s.height = 69;
			s.width = 49;
			s.length = 0.1;
			
			BLEND_ON();
			
			glColor4f(COLOR_GLASS, windowAlpha);
			drawCuboid(s);
			
			BLEND_OFF();
			
			pWindow = pWindow->next;
		}
	glPopMatrix();
}

/* INTERFACE
 * paint all the components, that are not yet painted,
 * then clean up.
 */
void flushComponents() {
	paintWindows();
	clearWindows();
}

/**************************************************************************/

/* INTERFACE
 * Render the object passed
 */
void renderComponent(Object component) {

	switch(component.modelID) {
		default:
			printf("---\n\nERROR IN components.h renderComponent - modelID unknown: %X\n\n---\n\n", component.modelID);
			break;
		case OBJ_FIELD:
			putFieldBy(component.properties);
			break;
		case OBJ_PIG:
			putPigBy(component.properties);
			break;
		case OBJ_HOUSE:
			putHouseBy(component.properties);
			break;
		case OBJ_FENCEPART:
			putFencepartBy(component.properties);
			break;
		case OBJ_GATEPART:
			putGatepartBy(component.properties);
			break;
		case OBJ_CARRIAGE:
			putCarriageBy(component.properties);
			break;
		case OBJ_WELLARM:
			putWellarmBy(component.properties);
			break;
		case OBJ_WELLPOLE:
			putWellpoleBy(component.properties);
			break;
		case OBJ_WELLPIT:
			putWellpitBy(component.properties);
			break;
		case OBJ_TROUGH:
			putTroughBy(component.properties);
			break;
	}

}

void putFieldBy(Properties properties) {

	// includes a hole under the wellpit

	Size s;
	GLfloat w, l, x, z, temp;

	glPushMatrix();
	
		glRotatef(properties.rotation.xAxis, 1.0, 0.0, 0.0);
		glRotatef(properties.rotation.yAxis, 0.0, 1.0, 0.0);
		glRotatef(properties.rotation.zAxis, 0.0, 0.0, 1.0);
		
		glTranslatef(properties.position.x, properties.position.y, properties.position.z);
		
		glColor3f(COLOR_FIELD);
		
		glPushMatrix();
			
			w = properties.size.width/2.0 + (wellpit.properties.position.x-wellpit.properties.size.width/2.0);
			x = w / 2.0 - properties.size.width/2.0;
			
			glTranslatef(x, 0.0, 0.0);
			
			s.width = w;
			s.height = properties.size.height;
			s.length = properties.size.length;
			
			drawCuboid(s);
		
		glPopMatrix();
		
		glPushMatrix();
			
			temp = w + wellpit.properties.size.width;
			w = properties.size.width - temp;
			x = w / 2.0 + temp - properties.size.width/2.0;
			
			glTranslatef(x, 0.0, 0.0);
			
			s.width = w;
			s.height = properties.size.height;
			s.length = properties.size.length;
			
			drawCuboid(s);
		
		glPopMatrix();
		
		glPushMatrix();
			
			l = properties.size.length/2.0 + (wellpit.properties.position.z-wellpit.properties.size.length/2.0);
			z = l / 2.0 - properties.size.length/2.0;
			
			glTranslatef(wellpit.properties.position.x, 0.0, z);
			
			s.width = wellpit.properties.size.width;
			s.height = properties.size.height;
			s.length = l;
			
			drawCuboid(s);
		
		glPopMatrix();
		
		glPushMatrix();
			
			temp = l + wellpit.properties.size.length;
			l = properties.size.length - temp;
			z = l / 2.0 + temp - properties.size.length/2.0;
			
			glTranslatef(wellpit.properties.position.x, 0.0, z);
			
			s.width = wellpit.properties.size.width;
			s.height = properties.size.height;
			s.length = l;
			
			drawCuboid(s);
		
		glPopMatrix();
		
	glPopMatrix();
}

void putPigBy(Properties properties) {

	Size s;
	
	glPushMatrix(); // pig
	
		glTranslatef(properties.position.x, properties.position.y+15.0, properties.position.z); // move it up, so that the bottom-center will be at the desired position
		
		glRotatef(properties.rotation.xAxis, 1.0, 0.0, 0.0);
		glRotatef(properties.rotation.yAxis, 0.0, 1.0, 0.0);
		glRotatef(properties.rotation.zAxis, 0.0, 0.0, 1.0);		
		
		/* test */
		//glScalef(6.0, 6.0, 6.0);
		
		//glColor3f(COLOR_WHITE);
		
		/* test */
		//drawCuboid(properties.size);
		
		glPushMatrix(); // body
			
			glTranslatef(0.0, 0.0, 6.0);
			
			s.width = 20;
			s.height = 20;
			s.length = 35;
			
			glColor3f(COLOR_PIG_BODY);
			
			drawSphere(s, 40, 20);
			
			glPushMatrix(); // head
			
				glTranslatef(0.0, 0.0, -25);
				
				glPushMatrix(); // skull
			
					glTranslatef(0.0, 0.0, 4.0);
					
					s.width = 22;
					s.height = 22;
					s.length = 16;
					
					glColor3f(COLOR_PIG_BODY);
					
					drawSphere(s, 20, 20);
					
					glPushMatrix(); // left eye
			
						glTranslatef(-4.0, 6.0, -6.0);
					
						s.width = 1;
						s.height = 1;
						s.length = 1;
						
						glColor3f(COLOR_PIG_EYE);
						
						drawSphere(s, 10, 10);
						
					glPopMatrix(); // left eye
					
					glPushMatrix(); // right eye
			
						glTranslatef(+4.0, 6.0, -6.0);
					
						s.width = 1;
						s.height = 1;
						s.length = 1;
						
						glColor3f(COLOR_PIG_EYE);
						
						drawSphere(s, 10, 10);
						
					glPopMatrix(); // right eye
					
					glPushMatrix(); // nose
			
						glRotatef(90.0, 1.0, 0.0, 0.0);
						glTranslatef(0.0, -10.0, 0.0);
					
						s.width = 7;
						s.height = 10;
						s.length = 5;
						
						glColor3f(COLOR_PIG_BODY);
						
						drawCylinder(s, 10);
						
						glPushMatrix(); // left nosehole
						    glTranslatef(-1.5, -5.0, 0.0);
						    
						    s.width = 1.0;
						    s.height = 0.2;
						    s.length = 1.0;
						    
						    glColor3f(COLOR_BLACK);
						    
						    drawCylinder(s, 10);
						glPopMatrix(); // left nosehole
						
						glPushMatrix(); // right nosehole
						    glTranslatef(+1.5, -5.0, 0.0);
						    
						    s.width = 1.0;
						    s.height = 0.2;
						    s.length = 1.0;
						    
						    glColor3f(COLOR_BLACK);
						    
						    drawCylinder(s, 10);
						glPopMatrix(); // right nosehole
						
					glPopMatrix(); // nose
					
					glPushMatrix(); // left ear
			
						glTranslatef(-5.0, +11.0, +0.0);
					
						s.width = 5;
						s.height = 8;
						s.length = 0.5;
						
						glColor3f(COLOR_PIG_BODY);
						
						drawPigEar(s);
						
					glPopMatrix(); // left ear
					
					glPushMatrix(); // right ear
					
						glTranslatef(+5.0, +11.0, +0.0);
					
						s.width = 5;
						s.height = 8;
						s.length = 0.5;
						
						glColor3f(COLOR_PIG_BODY);
						
						drawPigEar(s);
						
					glPopMatrix(); // right ear
					
				glPopMatrix(); // skull
				
			glPopMatrix(); // head
			
			glPushMatrix(); // front left foot
			
				glTranslatef(-5.0, -7.5, -11.0);
			
				s.width = 3;
				s.height = 15;
				s.length = 3;
				
				glColor3f(COLOR_PIG_BODY);
				
				drawCylinder(s, 10);
				
			glPopMatrix(); // front left foot
			
			glPushMatrix(); // front right foot
			
				glTranslatef(+5.0, -7.5, -11.0);
			
				s.width = 3;
				s.height = 15;
				s.length = 3;
				
				glColor3f(COLOR_PIG_BODY);
				
				drawCylinder(s, 10);
				
			glPopMatrix(); // front right foot
			
			glPushMatrix(); // rear left foot
			
				glTranslatef(-5.0, -7.5, +12.5);
			
				s.width = 3;
				s.height = 15;
				s.length = 3;
				
				glColor3f(COLOR_PIG_BODY);
				
				drawCylinder(s, 10);
				
			glPopMatrix(); // rear left foot
			
			glPushMatrix(); // rear right foot
			
				glTranslatef(+5.0, -7.5, +12.5);
			
				s.width = 3;
				s.height = 15;
				s.length = 3;
				
				glColor3f(COLOR_PIG_BODY);
				
				drawCylinder(s, 10);
				
			glPopMatrix(); // rear right foot
			
			glPushMatrix(); // tail
			
			    glTranslatef(0.0, 0.0, +20.0);
			
				s.width = 7;
				s.height = 7;
				s.length = 7;
				
				glColor3f(COLOR_PIG_BODY);
				
				drawPigTail(s);
			    
			glPopMatrix(); // tail
			
		glPopMatrix(); // body
		
	glPopMatrix(); // pig

}

void putHouseBy(Properties properties) {

	Size s;
	
	glPushMatrix(); // house
	
		glTranslatef(properties.position.x, properties.position.y+125.0, properties.position.z);
		
		glRotatef(properties.rotation.xAxis, 1.0, 0.0, 0.0);
		glRotatef(properties.rotation.yAxis, 0.0, 1.0, 0.0);
		glRotatef(properties.rotation.zAxis, 0.0, 0.0, 1.0);		
		
		/* test */
		//glScalef(2.0, 2.0, 2.0);
		
		//glColor3f(COLOR_WHITE);
		
		/* test */
		//drawCuboid(properties.size);
		
		glPushMatrix(); // base
		
			glColor3f(COLOR_FLOOR);
			glTranslatef(0.0, -120.0, 0.0);
			
			s.width = 300;
			s.length = 750;
			s.height = 10;
			
			glPushMatrix();
				glTranslatef(+375.0, 0.0, 0.0);
				drawCuboid(s);
			glPopMatrix();
			
			glPushMatrix();
				glTranslatef(-375.0, 0.0, 0.0);
				drawCuboid(s);
			glPopMatrix();
			
			s.width = 450;
			s.length = 300;
			s.height = 10;
			
			glPushMatrix();
				glTranslatef(0.0, 0.0, -225.0);
				drawCuboid(s);
			glPopMatrix();
		
		glPopMatrix(); // base
		
		glPushMatrix(); // walls
		
			//front
			glPushMatrix();
				glTranslatef(-450.0, -40.0, +370.0);
				putWallStdWith(WINDOW);
			glPopMatrix();
			
			glPushMatrix();
				glTranslatef(-300.0, -40.0, +370.0);
				putWallStdWith(DOOR);
			glPopMatrix();
			
			glPushMatrix();
				glTranslatef(+300.0, -40.0, +370.0);
				putWallStdWith(DOOR);
			glPopMatrix();
			
			glPushMatrix();
				glTranslatef(+450.0, -40.0, +370.0);
				putWallStdWith(WINDOW);
			glPopMatrix();
			
			//left outer edge
			glPushMatrix();
				glTranslatef(-520.0, -40.0, 300.0);
				glRotatef(90.0, 0.0, 1.0, 0.0);
				putWallStdWith(NOTHING);
			glPopMatrix();
			
			glPushMatrix();
				glTranslatef(-520.0, -40.0, 150.0);
				glRotatef(90.0, 0.0, 1.0, 0.0);
				putWallStdWith(WINDOW);
			glPopMatrix();
			
			glPushMatrix();
				glTranslatef(-520.0, -40.0, 0.0);
				glRotatef(90.0, 0.0, 1.0, 0.0);
				putWallStdWith(NOTHING);
			glPopMatrix();
			
			glPushMatrix();
				glTranslatef(-520.0, -40.0, -150.0);
				glRotatef(90.0, 0.0, 1.0, 0.0);
				putWallStdWith(NOTHING);
			glPopMatrix();
			
			glPushMatrix();
				glTranslatef(-520.0, -40.0, -300.0);
				glRotatef(90.0, 0.0, 1.0, 0.0);
				putWallStdWith(NOTHING);
			glPopMatrix();
			
			//right outer edge
			glPushMatrix();
				glTranslatef(+520.0, -40.0, 300.0);
				glRotatef(90.0, 0.0, 1.0, 0.0);
				putWallStdWith(NOTHING);
			glPopMatrix();
			
			glPushMatrix();
				glTranslatef(+520.0, -40.0, 150.0);
				glRotatef(90.0, 0.0, 1.0, 0.0);
				putWallStdWith(WINDOW);
			glPopMatrix();
			
			glPushMatrix();
				glTranslatef(+520.0, -40.0, 0.0);
				glRotatef(90.0, 0.0, 1.0, 0.0);
				putWallStdWith(NOTHING);
			glPopMatrix();
			
			glPushMatrix();
				glTranslatef(+520.0, -40.0, -150.0);
				glRotatef(90.0, 0.0, 1.0, 0.0);
				putWallStdWith(NOTHING);
			glPopMatrix();
			
			glPushMatrix();
				glTranslatef(+520.0, -40.0, -300.0);
				glRotatef(90.0, 0.0, 1.0, 0.0);
				putWallStdWith(NOTHING);
			glPopMatrix();
			
			//back outer edge
			glPushMatrix();
				glTranslatef(-450.0, -40.0, -370.0);
				putWallStdWith(NOTHING);
			glPopMatrix();
			
			glPushMatrix();
				glTranslatef(-300.0, -40.0, -370.0);
				putWallStdWith(NOTHING);
			glPopMatrix();
			
			glPushMatrix();
				glTranslatef(-150.0, -40.0, -370.0);
				putWallStdWith(WINDOW);
			glPopMatrix();
			
			glPushMatrix();
				glTranslatef(0.0, -40.0, -370.0);
				putWallStdWith(NOTHING);
			glPopMatrix();
			
			glPushMatrix();
				glTranslatef(+150.0, -40.0, -370.0);
				putWallStdWith(WINDOW);
			glPopMatrix();
			
			glPushMatrix();
				glTranslatef(+300.0, -40.0, -370.0);
				putWallStdWith(NOTHING);
			glPopMatrix();
			
			glPushMatrix();
				glTranslatef(+450.0, -40.0, -370.0);
				putWallStdWith(NOTHING);
			glPopMatrix();
			
			//left middle edge
			glPushMatrix();
				glTranslatef(-370.0, -40.0, 300.0);
				glRotatef(90.0, 0.0, 1.0, 0.0);
				putWallStdWith(DOOR);
			glPopMatrix();
			
			glPushMatrix();
				glTranslatef(-370.0, -40.0, 150.0);
				glRotatef(90.0, 0.0, 1.0, 0.0);
				putWallStdWith(WINDOW);
			glPopMatrix();
			
			glPushMatrix();
				glTranslatef(-370.0, -40.0, 0.0);
				glRotatef(90.0, 0.0, 1.0, 0.0);
				putWallStdWith(DOOR);
			glPopMatrix();
			
			glPushMatrix();
				glTranslatef(-370.0, -40.0, -150.0);
				glRotatef(90.0, 0.0, 1.0, 0.0);
				putWallStdWith(NOTHING);
			glPopMatrix();
			
			//right middle edge
			glPushMatrix();
				glTranslatef(+370.0, -40.0, 300.0);
				glRotatef(90.0, 0.0, 1.0, 0.0);
				putWallStdWith(DOOR);
			glPopMatrix();
			
			glPushMatrix();
				glTranslatef(+370.0, -40.0, 150.0);
				glRotatef(90.0, 0.0, 1.0, 0.0);
				putWallStdWith(WINDOW);
			glPopMatrix();
			
			glPushMatrix();
				glTranslatef(+370.0, -40.0, 0.0);
				glRotatef(90.0, 0.0, 1.0, 0.0);
				putWallStdWith(DOOR);
			glPopMatrix();
			
			glPushMatrix();
				glTranslatef(+370.0, -40.0, -150.0);
				glRotatef(90.0, 0.0, 1.0, 0.0);
				putWallStdWith(NOTHING);
			glPopMatrix();
			
			//back middle edge
			glPushMatrix();
				glTranslatef(-300.0, -40.0, -220.0);
				putWallStdWith(DOOR);
			glPopMatrix();
			
			glPushMatrix();
				glTranslatef(-150.0, -40.0, -220.0);
				putWallStdWith(WINDOW);
			glPopMatrix();
			
			glPushMatrix();
				glTranslatef(0.0, -40.0, -220.0);
				putWallStdWith(WINDOW);
			glPopMatrix();
			
			glPushMatrix();
				glTranslatef(+150.0, -40.0, -220.0);
				putWallStdWith(WINDOW);
			glPopMatrix();
			
			glPushMatrix();
				glTranslatef(+300.0, -40.0, -220.0);
				putWallStdWith(DOOR);
			glPopMatrix();
			
			//left inner edge
			glPushMatrix();
				glTranslatef(-230.0, -40.0, 290.0);
				glRotatef(90.0, 0.0, 1.0, 0.0);
				putWallStdWith(PORCH);
			glPopMatrix();
			
			glPushMatrix();
				glTranslatef(-230.0, -40.0, 140.0);
				glRotatef(90.0, 0.0, 1.0, 0.0);
				putWallStdWith(PORCH);
			glPopMatrix();
			
			glPushMatrix();
				glTranslatef(-230.0, -40.0, -10.0);
				glRotatef(90.0, 0.0, 1.0, 0.0);
				putWallStdWith(PORCH);
			glPopMatrix();
			
			//right inner edge
			glPushMatrix();
				glTranslatef(+230.0, -40.0, 290.0);
				glRotatef(90.0, 0.0, 1.0, 0.0);
				putWallStdWith(PORCH);
			glPopMatrix();
			
			glPushMatrix();
				glTranslatef(+230.0, -40.0, 140.0);
				glRotatef(90.0, 0.0, 1.0, 0.0);
				putWallStdWith(PORCH);
			glPopMatrix();
			
			glPushMatrix();
				glTranslatef(+230.0, -40.0, -10.0);
				glRotatef(90.0, 0.0, 1.0, 0.0);
				putWallStdWith(PORCH);
			glPopMatrix();
			
			//back inner edge
			glPushMatrix();
				glTranslatef(-150.0, -40.0, -80.0);
				putWallStdWith(PORCH);
			glPopMatrix();
			
			glPushMatrix();
				glTranslatef(0.0, -40.0, -80.0);
				putWallStdWith(PORCH);
			glPopMatrix();
			
			glPushMatrix();
				glTranslatef(+150.0, -40.0, -80.0);
				putWallStdWith(PORCH);
			glPopMatrix();
			
		glPopMatrix(); // walls
		
		glPushMatrix(); // roof
		
			glTranslatef(0.0, 110, 0.0);
			
			glColor3f(COLOR_WALL);
			
			s.width = 337;
			s.length = 787;
			s.height = 1;
			
			glPushMatrix();
				glTranslatef(+375.0, -74.5, 0.0);
				drawCuboid(s);
			glPopMatrix();
			
			glPushMatrix();
				glTranslatef(-375.0, -74.5, 0.0);
				drawCuboid(s);
			glPopMatrix();
			
			s.width = 487;
			s.length = 337;
			s.height = 1;
			
			glPushMatrix();
				glTranslatef(0.0, -74.5, -225.0);
				drawCuboid(s);
			glPopMatrix();
			
			glColor3f(COLOR_ROOF);
			
			s.width = 1090;
			s.length = 790;
			s.height = 150;
			
			drawRoof(s);
			
		glPopMatrix(); // roof
		
	glPopMatrix(); // house

}

void putFencepartBy(Properties properties) {

	Size s;
	
	glPushMatrix();
	
		glTranslatef(properties.position.x, properties.position.y+25, properties.position.z);
	
		glRotatef(properties.rotation.xAxis, 1.0, 0.0, 0.0);
		glRotatef(properties.rotation.yAxis, 0.0, 1.0, 0.0);
		glRotatef(properties.rotation.zAxis, 0.0, 0.0, 1.0);		
		
		/* test */
		//glScalef(4.0, 4.0, 4.0);
		
		//glColor3f(COLOR_WHITE);
		
		/* test */
		//drawCuboid(properties.size);
		
		glColor3f(COLOR_STD_WOOD);
		
		s.width = 5;
		s.length = 5;
		s.height = 50;
		
		glPushMatrix();
			glTranslatef(-50.0, 0.0, -2.5);
			drawCylinder(s, 10);
		glPopMatrix();
		
		glPushMatrix();
			glTranslatef(+50.0, 0.0, -2.5);
			drawCylinder(s, 10);
		glPopMatrix();
		
		s.width = 5;
		s.length = 5;
		s.height = 150;
		
		glPushMatrix();
			glTranslatef(0.0, -10.0, +2.5);
			glRotatef(90.0, 0.0, 0.0, 1.0);			
			drawCylinder(s, 10);
		glPopMatrix();
		
		glPushMatrix();
			glTranslatef(0.0, +13.0, +2.5);
			glRotatef(90.0, 0.0, 0.0, 1.0);		
			drawCylinder(s, 10);
		glPopMatrix();
		
	glPopMatrix();

}

void putGatepartBy(Properties properties) {

	Size s;
	
	glPushMatrix();
	
		glTranslatef(properties.position.x, properties.position.y+25, properties.position.z);
	
		glRotatef(properties.rotation.xAxis, 1.0, 0.0, 0.0);
		glRotatef(properties.rotation.yAxis, 0.0, 1.0, 0.0);
		glRotatef(properties.rotation.zAxis, 0.0, 0.0, 1.0);		
		
		glTranslatef(+37.5, 0.0, 0.0);
		
		/* test */
		//glScalef(4.0, 4.0, 4.0);
		
		//glColor3f(COLOR_WHITE);
		
		/* test */
		//drawCuboid(properties.size);
		
		glColor3f(COLOR_STD_WOOD);
		
		s.width = 8;
		s.length = 8;
		s.height = 70;
		
		glPushMatrix();
			glTranslatef(-35.0, 0.0, 0.0);
			drawCylinder(s, 10);
		glPopMatrix();
		
		s.width = 5;
		s.length = 5;
		s.height = 50;
		
		glPushMatrix();
			glTranslatef(+35.0, 0.0, 0.0);
			drawCuboid(s);
		glPopMatrix();
		
		s.width = 75;
		s.length = 5;
		s.height = 5;
		
		glPushMatrix();
			glTranslatef(0.0, +22.5, 0.0);
			drawCuboid(s);
		glPopMatrix();
		
		glPushMatrix();
			glTranslatef(0.0, -22.5, 0.0);
			drawCuboid(s);
		glPopMatrix();
		
		s.width = 5;
		s.length = 5;
		s.height = 90;
		
		glPushMatrix();
			glRotatef(60.0, 0.0, 0.0, 1.0);			
			drawCylinder(s, 10);
		glPopMatrix();
		
		glPushMatrix();
			glRotatef(-60.0, 0.0, 0.0, 1.0);		
			drawCylinder(s, 10);
		glPopMatrix();
		
	glPopMatrix();

}

void putCarriageBy(Properties properties) {

	Size s;
	
	glPushMatrix();
	
		glTranslatef(properties.position.x, properties.position.y+50, properties.position.z);
	
		glRotatef(properties.rotation.xAxis, 1.0, 0.0, 0.0);
		glRotatef(properties.rotation.yAxis, 0.0, 1.0, 0.0);
		glRotatef(properties.rotation.zAxis, 0.0, 0.0, 1.0);		
		
		/* test */
		//glScalef(6.0, 6.0, 6.0);
		
		//glColor3f(COLOR_WHITE);
		
		/* test */
		//drawCuboid(properties.size);
		
		glColor3f(COLOR_STD_WOOD);
		
		glPushMatrix(); // body
		
			glTranslatef(0.0, 0.0, -40.0);
			
			s.width = 50;
			s.length = 5;
			s.height = 50;
			
			glPushMatrix(); // rear left wheel
				glRotatef(90.0, 0.0, 1.0, 0.0);
				glTranslatef(-57.5, -25.0, -55.0);
				drawWheel(s);
			glPopMatrix();
			
			glPushMatrix(); // front left wheel
				glRotatef(90.0, 0.0, 1.0, 0.0);
				glTranslatef(-57.5, -25.0, +55.0);
				drawWheel(s);
			glPopMatrix();
			
			glPushMatrix(); // rear right wheel
				glRotatef(90.0, 0.0, 1.0, 0.0);
				glTranslatef(+57.5, -25.0, -55.0);
				drawWheel(s);
			glPopMatrix();
			
			glPushMatrix(); // front right wheel
				glRotatef(90.0, 0.0, 1.0, 0.0);
				glTranslatef(+57.5, -25.0, +55.0);
				drawWheel(s);
			glPopMatrix();
			
			s.width = 3;
			s.length = 3;
			s.height = 122;
			
			glPushMatrix(); // rear axle
				glRotatef(90.0, 0.0, 0.0, 1.0);
				glTranslatef(-25.0, 0.0, -57.5);
				drawCylinder(s, 10);
			glPopMatrix();
			
			glPushMatrix(); // front axle
				glRotatef(90.0, 0.0, 0.0, 1.0);
				glTranslatef(-25.0, 0.0, +57.5);
				drawCylinder(s, 10);
			glPopMatrix();
			
			s.width = 100;
			s.length = 170;
			s.height = 2;
			
			glPushMatrix(); // bottom
				glTranslatef(0.0, -22.5, 0.0);
				drawCuboid(s);
			glPopMatrix();
			
			s.width = 2;
			s.length = 170;
			s.height = 72.5;
			
			glPushMatrix(); // left side
				glTranslatef(-49.0, +13.75, 0.0);
				drawCuboid(s);
			glPopMatrix();
			
			glPushMatrix(); // right side
				glTranslatef(+49.0, +13.75, 0.0);
				drawCuboid(s);
			glPopMatrix();
			
			s.width = 100;
			s.length = 2;
			s.height = 72.5;
			
			glPushMatrix(); // back side
				glTranslatef(0.0, +13.75, -84.0);
				drawCuboid(s);
			glPopMatrix();
			
			glPushMatrix(); // front side
				glTranslatef(0.0, +13.75, +84.0);
				drawCuboid(s);
			glPopMatrix();
			
		glPopMatrix(); // body
		
		glPushMatrix(); // rod
		
			glRotatef(90.0, 1.0, 0.0, 0.0);
			glTranslatef(0.0, +85.0, +10.0);
			
			s.width = 5;
			s.length = 5;
			s.height = 80;
			
			drawCylinder(s, 10);
			
			s.width = 3;
			s.length = 3;
			s.height = 80;
			
			glRotatef(90.0, 0.0, 0.0, 1.0);
			glTranslatef(+25.0, 0.0, 0.0);
			
			drawCylinder(s, 10);
		
		glPopMatrix(); // rod
		
	glPopMatrix();

}

void putWellarmBy(Properties properties) {

	// it haz a bukket!

	Size s;
	
	glPushMatrix();
	
		glTranslatef(properties.position.x, properties.position.y+2.5, properties.position.z);
	
		glPushMatrix(); // arm
			glRotatef(properties.rotation.xAxis-90.0, 1.0, 0.0, 0.0);
			glRotatef(properties.rotation.yAxis, 0.0, 1.0, 0.0);
			glRotatef(properties.rotation.zAxis, 0.0, 0.0, 1.0);		
			
			glColor3f(COLOR_STD_WOOD);
			
			s.width = 5;
			s.length = 5;
			s.height = 160;
			
			drawCylinder(s, 10);
			
			s.width = 3;
			s.length = 3;
			s.height = 110;
		glPopMatrix(); // arm
		
		glPushMatrix(); // components
			glTranslatef(0.0, 80*cos( ((properties.rotation.xAxis-90.0) / 180.0) * GL_PI )-55.0, 80*sin( ((properties.rotation.xAxis-90.0) / 180.0) * GL_PI )+2.0);
			drawCylinder(s, 10);
		
			glPushMatrix(); // bucket
				glTranslatef(0.0, -67.5, 0.0);
				
				s.width = 0.75;
				s.length = 0.75;
				s.height = 11.2;
				
				glPushMatrix(); // handle
				
					glRotatef(-25.0, 0.0, 0.0, 1.0);
					glTranslatef(-5.0, +5.5, 0.0);
					
					glColor3f(COLOR_METAL);
					
					drawCylinder(s, 6);
				
				glPopMatrix();
				
				glPushMatrix(); // handle
				
					glRotatef(+25.0, 0.0, 0.0, 1.0);
					glTranslatef(+5.0, +5.5, 0.0);
					
					glColor3f(COLOR_METAL);
					
					drawCylinder(s, 6);
				
				glPopMatrix();
				
				glPushMatrix(); // body
					glTranslatef(0.0, -5.0, 0.0);
					
					glColor3f(COLOR_METAL);
				
					s.width = 10;
					s.length = 10;
					s.height = 15;
					
					drawCubicTorus(s, 0.5, 20);
					
					s.width = 10;
					s.length = 10;
					s.height = 0.5;
					
					glPushMatrix(); // base circle
						glTranslatef(0.0, -8, 0.0);
						drawCylinder(s, 20);
					glPopMatrix();
					
					s.width = 9.5;
					s.length = 9.5;
					s.height = 7;
					
					BLEND_ON();
					
					glColor4f(COLOR_WATER, waterAlpha); // water
					
					glPushMatrix();
						glTranslatef(0.0, -3.0, 0.0);
						drawCylinder(s, 20);
					glPopMatrix();
					
					BLEND_OFF();
					
				glPopMatrix(); // body
			glPopMatrix(); // bucket
		glPopMatrix(); // components
		
	glPopMatrix();

}

void putWellpoleBy(Properties properties) {

	Size s;
	
	glPushMatrix();
	
		glTranslatef(properties.position.x, properties.position.y+47.5, properties.position.z);
	
		glRotatef(properties.rotation.xAxis, 1.0, 0.0, 0.0);
		glRotatef(properties.rotation.yAxis, 0.0, 1.0, 0.0);
		glRotatef(properties.rotation.zAxis, 0.0, 0.0, 1.0);		
		
		/* test */
		//glScalef(4.0, 4.0, 4.0);
		
		//glColor3f(COLOR_WHITE);
		
		/* test */
		//drawCuboid(properties.size);
		
		glColor3f(COLOR_STD_WOOD);
		
		s.width = 5;
		s.length = 5;
		s.height = 95;
		
		drawCylinder(s, 10);
		
	glPopMatrix();

}

void putWellpitBy(Properties properties) {

	Size s;
	
	glPushMatrix();
	
		glTranslatef(properties.position.x, properties.position.y-50, properties.position.z);
	
		glRotatef(properties.rotation.xAxis, 1.0, 0.0, 0.0);
		glRotatef(properties.rotation.yAxis, 0.0, 1.0, 0.0);
		glRotatef(properties.rotation.zAxis, 0.0, 0.0, 1.0);		
		
		/* test */
		//glScalef(4.0, 4.0, 4.0);
		
		//glColor3f(COLOR_WHITE);
		
		/* test */
		//drawCuboid(properties.size);
		
		glColor3f(COLOR_STD_WOOD);
		
		s.width = 2;
		s.length = 75;
		s.height = 200;
		
		glPushMatrix();
			glTranslatef(-36.5, 0.0, 0.0);
			drawCuboid(s);
		glPopMatrix();
		
		glPushMatrix();
			glTranslatef(+36.5, 0.0, 0.0);
			drawCuboid(s);
		glPopMatrix();
		
		s.width = 75;
		s.length = 2;
		s.height = 200;
		
		glPushMatrix();
			glTranslatef(0.0, 0.0, -36.5);
			drawCuboid(s);
		glPopMatrix();
		
		glPushMatrix();
			glTranslatef(0.0, 0.0, +36.5);
			drawCuboid(s);
		glPopMatrix();
		
		s.width = 75;
		s.length = 75;
		s.height = 2;
		
		glColor3f(COLOR_BLACK);
		
		glPushMatrix();
			glTranslatef(0.0, -99.0, 0.0);
			drawCuboid(s);
		glPopMatrix();
		
		s.width = 73;
		s.length = 73;
		s.height = 150;
		
		BLEND_ON();
		
		glColor4f(COLOR_WATER, waterAlpha);
		
		glPushMatrix(); // water
			glTranslatef(0.0, -73.0, 0.0);
			drawCuboid(s);
		glPopMatrix();
		
		BLEND_OFF();
		
	glPopMatrix();

}

void putTroughBy(Properties properties) {

	Size s;
	
	glPushMatrix();
	
		glTranslatef(properties.position.x, properties.position.y+10, properties.position.z);
	
		glRotatef(properties.rotation.xAxis, 1.0, 0.0, 0.0);
		glRotatef(properties.rotation.yAxis, 0.0, 1.0, 0.0);
		glRotatef(properties.rotation.zAxis, 0.0, 0.0, 1.0);		
		
		/* test */
		//glScalef(4.0, 4.0, 4.0);
		
		//glColor3f(COLOR_WHITE);
		
		/* test */
		//drawCuboid(properties.size);
		
		glColor3f(COLOR_STD_WOOD);
		
		s.width = 2;
		s.length = 100;
		s.height = 20;
		
		glPushMatrix();
			glTranslatef(-14.0, 0.0, 0.0);
			drawCuboid(s);
		glPopMatrix();
		
		glPushMatrix();
			glTranslatef(+14.0, 0.0, 0.0);
			drawCuboid(s);
		glPopMatrix();
		
		s.width = 30;
		s.length = 2;
		s.height = 20;
		
		glPushMatrix();
			glTranslatef(0.0, 0.0, -49.0);
			drawCuboid(s);
		glPopMatrix();
		
		glPushMatrix();
			glTranslatef(0.0, 0.0, +49.0);
			drawCuboid(s);
		glPopMatrix();
		
		s.width = 30;
		s.length = 100;
		s.height = 2;
		
		glPushMatrix();
			glTranslatef(0.0, -9.0, 0.0);
			drawCuboid(s);
		glPopMatrix();
		
		s.width = 28;
		s.length = 98;
		s.height = 10;
		
		BLEND_ON();
		
		glColor4f(COLOR_WATER, waterAlpha);
		
		glPushMatrix();
			glTranslatef(0.0, -3.0, 0.0);
			drawCuboid(s);
		glPopMatrix();
		
		BLEND_OFF();
		
	glPopMatrix();

}


void putWallStdWith(int what) {
	Size s;
	
	switch(what) {
		default : /* fallthrough */
		case NOTHING :
			glColor3f(COLOR_WALL);
			
			s.height = 150;
			s.width = 150;
			s.length = 10;
			
			drawCuboid(s);
			break;
			
		case WINDOW :
			glColor3f(COLOR_WALL);
			
			s.height = 150;
			s.width = 50;
			s.length = 10;
			
			glPushMatrix(); // left wall
				glTranslatef(-50.0, 0.0, 0.0);
				drawCuboid(s);
			glPopMatrix();
			
			glPushMatrix(); // right wall
				glTranslatef(+50.0, 0.0, 0.0);
				drawCuboid(s);
			glPopMatrix();
			
			s.height = 30;
			s.width = 150;
			s.length = 10;
			
			glPushMatrix(); // top wall
				glTranslatef(0.0, +60.0, 0.0);
				drawCuboid(s);
			glPopMatrix();
			
			s.height = 50;
			s.width = 150;
			s.length = 10;
			
			glPushMatrix(); // bottom wall
				glTranslatef(0.0, -50.0, 0.0);
				drawCuboid(s);
			glPopMatrix();
			
			glPushMatrix(); // window
				glTranslatef(0.0, +10.0, 0.0);
				
				s.height = 70;
				s.width = 4;
				s.length = 2;
				
				glColor3f(COLOR_PNT_WOOD);
				
				glPushMatrix(); // left wood
					glTranslatef(-23.0, 0.0, 0.0);
					drawCuboid(s);
				glPopMatrix();
				glPushMatrix(); // middle wood
					drawCuboid(s);
				glPopMatrix();
				glPushMatrix(); // right wood
					glTranslatef(+23.0, 0.0, 0.0);
					drawCuboid(s);
				glPopMatrix();
				
				s.height = 4;
				s.width = 50;
				s.length = 2;
				
				glPushMatrix(); // top wood
					glTranslatef(0.0, +33.0, 0.0);
					drawCuboid(s);
				glPopMatrix();
				glPushMatrix(); // middle wood
					drawCuboid(s);
				glPopMatrix();
				glPushMatrix(); // bottom wood
					glTranslatef(0.0, -33.0, 0.0);
					drawCuboid(s);
				glPopMatrix();
				
				GLdouble * modelMatrix = (GLdouble *)malloc(16 * sizeof(GLdouble)); // glass
				glGetDoublev(GL_MODELVIEW, modelMatrix);
				addWindow(modelMatrix); // save it for later painting
				
			glPopMatrix();
			break;
			
		case DOOR :
			glColor3f(COLOR_WALL);
			
			s.height = 150;
			s.width = 30;
			s.length = 10;
			
			glPushMatrix(); // left wall
				glTranslatef(-60.0, 0.0, 0.0);
				drawCuboid(s);
			glPopMatrix();
			
			glPushMatrix(); // right wall
				glTranslatef(+60.0, 0.0, 0.0);
				drawCuboid(s);
			glPopMatrix();
			
			s.height = 30;
			s.width = 150;
			s.length = 10;
			
			glPushMatrix(); // top wall
				glTranslatef(0.0, +60.0, 0.0);
				drawCuboid(s);
			glPopMatrix();
			
			glPushMatrix(); // door
				glTranslatef(0.0, -15.0, 0.0);
				
				s.height = 120; // door
				s.width = 90;
				s.length = 2;
				glColor3f(COLOR_PNT_WOOD);
				drawCuboid(s);
				
				s.height = 8;
				s.width = 2;
				s.length = 2;
				
				glColor3f(COLOR_METAL);
				glTranslatef(-42.0, -10.0, 0.0);
				
				glPushMatrix(); // handle axle
					glRotatef(90.0, 1.0, 0.0, 0.0);
					drawCylinder(s, 8);
				glPopMatrix();
				glPushMatrix(); // handle 1
					glRotatef(90.0, 0.0, 0.0, 1.0);
					glTranslatef(0.0, -4.0, +4);
					drawCylinder(s, 8);
				glPopMatrix();
				glPushMatrix(); // handle 2
					glRotatef(90.0, 0.0, 0.0, 1.0);
					glTranslatef(0.0, -4.0, -4);
					drawCylinder(s, 8);
				glPopMatrix();
			glPopMatrix();
			break;
			
		case PORCH :
			glColor3f(COLOR_WALL);
			//glColor3f(0.8, 0.8, 0.8);
			
			s.height = 150;
			s.width = 10;
			s.length = 10;
			
			glPushMatrix(); // left wall
				glTranslatef(-70.0, 0.0, 0.0);
				drawCuboid(s);
			glPopMatrix();
			
			glPushMatrix(); // right wall
				glTranslatef(+70.0, 0.0, 0.0);
				drawCuboid(s);
			glPopMatrix();
			
			s.height = 10;
			s.width = 150;
			s.length = 10;
			
			glPushMatrix(); // top wall
				glTranslatef(0.0, +70.0, 0.0);
				drawCuboid(s);
			glPopMatrix();
			
			glPushMatrix(); // arch
				glTranslatef(0.0, +32.5, 0.0);
				
				s.height = 65;
				s.width = 130;
				s.length = 10;
				
				drawArch(s, 20);
			glPopMatrix();
			break;
	}
}
