#ifndef COMPONENTS_H
#define COMPONENTS_H

#include "global.h"

typedef struct {
	GLfloat	redChannel;
	GLfloat	greenChannel;
	GLfloat	blueChannel;
} Color;

typedef struct {
	GLfloat	x;
	GLfloat	y;
	GLfloat	z;
} Position;

typedef struct {
	GLfloat	xAxis;
	GLfloat	yAxis;
	GLfloat	zAxis;
} Rotation;

typedef struct {
	GLfloat	height;
	GLfloat width;
	GLfloat length;
} Size;

typedef struct {
	Position		position;	// position of bottom-center relative to parent
	Rotation		rotation;	// absolute rotation relative to parent
	Size			size;		// the height/width of the smallest cuboid this object could be put into. Can be read for convenient placement computation and stuff, BUT it might be inaccurate. Do not modify
} Properties;

typedef struct {
	unsigned int	modelID;
	Properties		properties;
} Object;


/*
 * Render the object passed
 */
void renderComponent(Object component);

/*
 * paint all the components, that are not yet painted,
 * then clean up.
 */
void flushComponents();

/*
 * toplevel components used for building the map. Their parent must be the map.
 * Position and rotation must be set explicitly before use.
 * If more than one piece is needed of an object (e.g. fence- or gateparts),
 *		create a copy of them and use that.
 *
 * WARNING: animations.c might fiddle with the positions and rotations of some of these objects
 */
extern Object field; // properties.size can be set to scale
extern Object pig;
extern Object house; 
extern Object fencepart; // subcomponent. Create a copy to use
extern Object gatepart; // subcomponent. Create a copy to use
extern Object carriage;
extern Object wellarm;
extern Object wellpole;
extern Object wellpit;
extern Object trough; // HU:vly

extern Object fence[]; // virtual component, copy of a subcomponent
extern Object gate_l; // virtual component, copy of a subcomponent
extern Object gate_r; // virtual component, copy of a subcomponent

extern double windowAlpha;
extern double waterAlpha;

extern boolean spotLightsOn;
extern boolean sunOn;

#endif /* COMPONENTS_H */