#include "map.h"

#include "components.h"

int bounds[FIELDSIZE/AREASIZE][FIELDSIZE/AREASIZE]; // bounds - default for closed gates, mask with openExtraBounds to toggle between open and closed
int openExtraBounds[FIELDSIZE/AREASIZE][FIELDSIZE/AREASIZE]; // extra free bounds for open gates // open: bounds &= open close: bounds ^= open

/* INTERFACE
 * Build the map from the components. Must be invoked once, in the initialisation state. Can be used for resetting as well
 */
void buildMap() {
	
	int i, j;
	
	field.properties.position.x = 0.0;
	field.properties.position.y = -120.0;
	field.properties.position.z = 0.0;
	
	
	pig.properties.position.x = -30.0;
	pig.properties.position.y = -95.0;
	pig.properties.position.z = -160.0;
	
	pig.properties.rotation.xAxis = 0.0;
	pig.properties.rotation.yAxis = -90.0;
	pig.properties.rotation.zAxis = 0.0;
	
	
	house.properties.position.x = 0.0;
	house.properties.position.y = -95.0;
	house.properties.position.z = -200.0;
	
	
	// init fence
	for(i = 0; i < FENCECOUNT; i++) { // fencecount in map.h
		fence[i].modelID = fencepart.modelID;
		
		fence[i].properties.position.x = fencepart.properties.position.x;
		fence[i].properties.position.y = fencepart.properties.position.y;
		fence[i].properties.position.z = fencepart.properties.position.z;
		
		fence[i].properties.rotation.xAxis = fencepart.properties.rotation.xAxis;
		fence[i].properties.rotation.yAxis = fencepart.properties.rotation.yAxis;
		fence[i].properties.rotation.zAxis = fencepart.properties.rotation.zAxis;
		
		fence[i].properties.size.height = fencepart.properties.size.height;
		fence[i].properties.size.width = fencepart.properties.size.width;
		fence[i].properties.size.length = fencepart.properties.size.length;
	}
	
	{
		GLfloat x;
	
		// yard fence
		fence[0].properties.position.x = -150.0;
		fence[0].properties.position.y = -95.0;
		fence[0].properties.position.z = +170.0;
		
		fence[1].properties.position.x = +150.0;
		fence[1].properties.position.y = -95.0;
		fence[1].properties.position.z = +170.0;
		
		// house-side fence
		fence[2].properties.position.x = -600.0;
		fence[2].properties.position.y = -95.0;
		fence[2].properties.position.z = +170.0;
		
		fence[3].properties.position.x = +600.0;
		fence[3].properties.position.y = -95.0;
		fence[3].properties.position.z = +170.0;
		
		// opposite fence
		x = -600.0;
		for(i = 4; i <= 12; i++, x += 150.0) {
			fence[i].properties.position.x = x;
			fence[i].properties.position.y = -95.0;
			fence[i].properties.position.z = +650.0;
			
			fence[i].properties.rotation.yAxis += 180.0;
		}
	}
	
	
	gate_l.modelID = gatepart.modelID;
		
	gate_l.properties.position.x = -75.0;
	gate_l.properties.position.y = -95.0;
	gate_l.properties.position.z = +172.5;
	
	gate_l.properties.rotation.xAxis = gatepart.properties.rotation.xAxis;
	gate_l.properties.rotation.yAxis = gatepart.properties.rotation.yAxis;
	gate_l.properties.rotation.zAxis = gatepart.properties.rotation.zAxis;
	
	gate_l.properties.size.height = gatepart.properties.size.height;
	gate_l.properties.size.width = gatepart.properties.size.width;
	gate_l.properties.size.length = gatepart.properties.size.length;
	
	gate_r.modelID = gatepart.modelID;
		
	gate_r.properties.position.x = +75.0;
	gate_r.properties.position.y = -95.0;
	gate_r.properties.position.z = +172.5;
	
	gate_r.properties.rotation.xAxis = gatepart.properties.rotation.xAxis;
	gate_r.properties.rotation.yAxis = gatepart.properties.rotation.yAxis+180.0;
	gate_r.properties.rotation.zAxis = gatepart.properties.rotation.zAxis;
	
	gate_r.properties.size.height = gatepart.properties.size.height;
	gate_r.properties.size.width = gatepart.properties.size.width;
	gate_r.properties.size.length = gatepart.properties.size.length;
	
	
	carriage.properties.position.x = +145.0;
	carriage.properties.position.y = -95.0;
	carriage.properties.position.z = -100.0;
	
	
	wellpole.properties.position.x = -60.0;
	wellpole.properties.position.y = -95.0;
	wellpole.properties.position.z = +50.0;
	
	
	wellarm.properties.position.x = -55.0;
	wellarm.properties.position.y = -10.0;
	wellarm.properties.position.z = +40.0;
	
	wellarm.properties.rotation.xAxis = 45.0;
	wellarm.properties.rotation.yAxis = 0.0;
	wellarm.properties.rotation.zAxis = 0.0;
	
	
	wellpit.properties.position.x = -50.0;
	wellpit.properties.position.y = -95.0;
	wellpit.properties.position.z = -20.0;
	
	
	trough.properties.position.x = -37.5;
	trough.properties.position.y = -95.0;
	trough.properties.position.z = -77.5;
	
	trough.properties.rotation.xAxis = 0.0;
	trough.properties.rotation.yAxis = 90.0;
	trough.properties.rotation.zAxis = 0.0;
	
	
	for(i = 0; i < FIELDSIZE/AREASIZE; i++) {
		for(j = 0; j < FIELDSIZE/AREASIZE; j++) {
			openExtraBounds[i][j] = 0;
			bounds[i][j] = 0;
			if( ( (j == 10 || j == 15) && (i >= 10 && i <= 14) ) || ((i == 10 || i == 14) && (j >= 10 && j <= 15)) || ((j >= 18 && j <= 25) && i >= 1) ) {
				bounds[i][j] = 1;
			}
		}
	}
	
	openExtraBounds[12][17] = 1;
	openExtraBounds[13][17] = 1;
	openExtraBounds[14][17] = 1;
	openExtraBounds[12][16] = 1;
	openExtraBounds[13][16] = 1;
	openExtraBounds[14][16] = 1;
}

/* INTERFACE
 * Draw the map. This is the main drawing function.
 */
void drawMap() {
	
	int i;
	
	renderComponent(field);
	renderComponent(pig);
	renderComponent(house);
	
	for(i = 0; i < FENCECOUNT; i++) { // fencecount in map.h
		renderComponent(fence[i]);
	}
	
	renderComponent(gate_l);
	renderComponent(gate_r);
	renderComponent(carriage);
	renderComponent(wellarm);
	renderComponent(wellpole);
	renderComponent(wellpit);
	renderComponent(trough);
	
	flushComponents();

}

/* INTERFACE
 * set the bounds of the pig's movements for open gates
 */
void setBoundsOpenGates() {
	int i, j;
	
	for(i = 0; i < FIELDSIZE/AREASIZE; i++) {
		for(j = 0; j < FIELDSIZE/AREASIZE; j++) {
			if(openExtraBounds[i][j] == 1)
				bounds[i][j] = 1;
		}
	}
}

/* INTERFACE
 * set the bounds of the pig's movements for closed gates
 */
void setBoundsClosedGates() {
	int i, j;
	
	for(i = 0; i < FIELDSIZE/AREASIZE; i++) {
		for(j = 0; j < FIELDSIZE/AREASIZE; j++) {
			if(openExtraBounds[i][j] == 1)
				bounds[i][j] = 0;
		}
	}
}
